/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.app.Activity
import com.google.gson.Gson
import io.flutter.plugin.common.MethodCall
import io.scanbot.sdk.flutter.utils.ObjectMapper
import io.scanbot.sdk.ui.view.barcode.configuration.BarcodeScannerConfiguration
import io.scanbot.sdk.ui.view.base.configuration.CameraOrientationMode
import io.scanbot.sdk.ui.view.base.configuration.OrientationMode
import io.scanbot.sdk.ui.view.camera.configuration.DocumentScannerConfiguration
import io.scanbot.sdk.ui.view.edit.configuration.CroppingConfiguration
import io.scanbot.sdk.ui.view.hic.configuration.HealthInsuranceCardScannerConfiguration
import io.scanbot.sdk.ui.view.mrz.configuration.MRZScannerConfiguration
import io.scanbot.sdk.barcode.entity.BarcodeFormat


fun readEhicScannerConfiguration(
        call: MethodCall,
        activity: Activity
): HealthInsuranceCardScannerConfiguration {
    val cameraConfiguration = HealthInsuranceCardScannerConfiguration()
    val arguments = call.arguments as EhicScannerConfigurationJson
    val value = checkCameraOrientationParam(call, arguments)
    value?.let {
        cameraConfiguration.setOrientationLockMode(value)
    }
    ObjectMapper.map(arguments, cameraConfiguration)
    return cameraConfiguration
}

fun readBarcodeScannerConfiguration(
        call: MethodCall,
        activity: Activity
): BarcodeScannerConfiguration {
    val barcodeCameraConfiguration = BarcodeScannerConfiguration()
    val arguments = call.arguments as BarcodeScannerConfigurationJson
    val value = checkCameraOrientationParam(call, arguments)
    value?.let {
        barcodeCameraConfiguration.setOrientationLockMode(value)
    }
    ObjectMapper.map(arguments, barcodeCameraConfiguration)

    if (arguments.containsKey("barcodeFormats")) {
        val barcodeFormats = arguments["barcodeFormats"] as ArrayList<String>
        val nativeBarcodeFormats = ArrayList<BarcodeFormat>()
        for (formatValue in barcodeFormats) {
            nativeBarcodeFormats.add(BarcodeFormat.valueOf(formatValue.replaceFirst("BarcodeFormat.", "")))
        }
        barcodeCameraConfiguration.setBarcodeFormatsFilter(nativeBarcodeFormats)
    }

    return barcodeCameraConfiguration
}

fun readCroppingConfiguration(
        call: MethodCall,
        activity: Activity,
        gson: Gson
): CroppingConfiguration {
    val croppingConfiguration = CroppingConfiguration()
    val arguments = call.arguments as CroppingConfigurationJson
    call.argument<PageJson>("page")?.let {
        val page = it.toPage(gson = gson)
        croppingConfiguration.setPage(page)
        arguments.remove("page")
    }
    val value = checkOrientationParam(call, arguments)
    value?.let {
        croppingConfiguration.setOrientationLockMode(value)
    }
    ObjectMapper.map(arguments, croppingConfiguration)
    return croppingConfiguration
}

fun readMRZScannerConfiguration(
        call: MethodCall,
        activity: Activity
): MRZScannerConfiguration {
    val cameraConfiguration = MRZScannerConfiguration()
    val arguments = call.arguments as MRZScannerConfigurationJson
    val value = checkCameraOrientationParam(call, arguments)
    value?.let {
        cameraConfiguration.setOrientationLockMode(value)
    }
    ObjectMapper.map(arguments, cameraConfiguration)
    return cameraConfiguration
}

private fun checkCameraOrientationParam(call: MethodCall, arguments: MRZScannerConfigurationJson): CameraOrientationMode? {
    if (call.hasArgument("orientationLockMode")) {
        val value = when (call.argument<String>("orientationLockMode")) {
            "PORTRAIT" -> CameraOrientationMode.PORTRAIT
            "PORTRAIT_UPSIDE_DOWN" -> CameraOrientationMode.PORTRAIT
            "LANDSCAPE_LEFT" -> CameraOrientationMode.LANDSCAPE
            "LANDSCAPE_RIGHT" -> CameraOrientationMode.LANDSCAPE
            "LANDSCAPE" -> CameraOrientationMode.LANDSCAPE
            else -> CameraOrientationMode.PORTRAIT
        }
        arguments.remove("orientationLockMode")
        return value
    }
    return null
}
private fun checkOrientationParam(call: MethodCall, arguments: MRZScannerConfigurationJson): OrientationMode? {
    if (call.hasArgument("orientationLockMode")) {
        val value = when (call.argument<String>("orientationLockMode")) {
            "PORTRAIT" -> OrientationMode.PORTRAIT
            "PORTRAIT_UPSIDE_DOWN" -> OrientationMode.PORTRAIT
            "LANDSCAPE_LEFT" -> OrientationMode.LANDSCAPE
            "LANDSCAPE_RIGHT" -> OrientationMode.LANDSCAPE
            "LANDSCAPE" -> OrientationMode.LANDSCAPE
            else -> OrientationMode.PORTRAIT
        }
        arguments.remove("orientationLockMode")
        return value
    }
    return null
}

fun readDocumentScannerConfiguration(
        call: MethodCall,
        activity: Activity
): DocumentScannerConfiguration {
    val cameraConfiguration = DocumentScannerConfiguration()
    val arguments = call.arguments as DocumentScannerConfigurationJson
    val value = checkCameraOrientationParam(call, arguments)
    value?.let {
        cameraConfiguration.setOrientationLockMode(value)
    }
    ObjectMapper.map(arguments, cameraConfiguration)
    return cameraConfiguration
}


