//
//  Renderer.swift
//  Pods-Runner
//
//  Created by Zhenya Knizhnik on 02.09.19.
//

import Foundation
import ScanbotSDK
import Flutter

final class Renderer {
    func renderPDF(from pages: [SBSDKUIPage],
                   pageSize: SBSDKPDFRendererPageSize = .fromImage,
                   result: @escaping FlutterResult) {
        if pages.isEmpty {
            result(FlutterError(code: "1010", message: "ERROR_iOS: No pages passed", details: "Rendering PDF error"))
            return
        }
        
        DispatchQueue.main.async {
            let storage = SBSDKIndexedImageStorage.temporaryStorage(for: pages)
            let outputURL = PluginPreferences.shared.newStorageCustomFileURL(extensionString: "pdf")
            // TODO eddy: change to SBSDKUIPDFRenderer
            SBSDKPDFRenderer.renderImageStorage(storage,
                                                copyImageStorage: false,
                                                indexSet: nil,
                                                with: pageSize,
                                                output: outputURL,
                                                completionHandler: { (finished, error, info) in
                self.pdfProcessCompletion(finished: finished, error: error, info: info, result: result)
            })
        }
    }
    
    private func pdfProcessCompletion(finished: Bool,
                                      error: Error?,
                                      info: [String: NSObject]?,
                                      result: @escaping FlutterResult) {
        if let error = error {
            result(FlutterError(code: "1006", message: "ERROR_iOS: " + error.localizedDescription, details: "Rendering PDF error"))
        } else if finished, let outputURL = info?[SBSDKResultInfoDestinationFileURLKey] as? URL {
            result(outputURL.absoluteString)
        } else {
            result(FlutterError(code: "1007", message: "ERROR_iOS: Unknown rendering error", details: "Rendering PDF error"))
        }
    }
    
    func renderTIFF(from pages: [SBSDKUIPage],
                    options: TIFFOptions,
                    result: @escaping FlutterResult) {
        
        if pages.isEmpty {
            result(FlutterError(code: "1011", message: "ERROR_iOS: No pages passed", details: "Rendering TIFF error"))
            return
        }
        DispatchQueue.main.async {
            let outputURL = PluginPreferences.shared.newStorageCustomFileURL(extensionString: "tiff")
            let imageURLs = pages.compactMap({ $0.documentImageURL() })
            
            let params = SBSDKTIFFImageWriterParameters.default()

            params.dpi = options.dpi
            params.binarize = options.isBinarized
            params.compression = options.compression

            let success = SBSDKTIFFImageWriter.writeTIFF(from: imageURLs, fileURL: outputURL, parameters: params)
            
            if success {
                result(outputURL.absoluteString)
            } else {
                result(FlutterError(code: "1008", message: "ERROR_iOS: Unknown rendering error", details: "Rendering TIFF error"))
            }
        }
    }
}
