import 'dart:convert';
import 'dart:typed_data';
import 'dart:ui';

import 'package:flutter/painting.dart';
import 'package:scanbot_sdk/scanbot_sdk_models.dart';

import 'barcode_scanning_data.dart';
import 'common_data.dart';

void addColor(Color color, String name, Map arguments) {
  if (color != null) arguments[name] = toColorString(color);
}

void addParam(dynamic argument, String name, Map arguments) {
  if (argument != null) arguments[name] = argument;
}

String toColorString(Color color) {
  return color != null ? '#${color.value.toRadixString(16)}' : null;
}

Color colorFromString(String color) {
  String valueString = color.replaceAll("#", "");
  int value = int.parse(valueString, radix: 16);
  return new Color(value);
}

Uri stringToFileUri(String string) {
  return Uri.parse(string);
}

String fileUriToString(Uri uri) {
  return uri.toString();
}

DateTime dateFromString(String string) {
  if (string == null) return null;
  return DateTime.parse(string);
}

Uint8List imageFromBase64String(String string) {
  return base64Decode(string);
}

String imageToBase64String(Uint8List image) {
  return base64Encode(image);
}

List<String> barcodeFormatsToJson(List<BarcodeFormat> barcodeFormats) {
  return barcodeFormats?.map((e) => _$BarcodeFormatEnumMap[e])?.toList();
}

const _$BarcodeFormatEnumMap = {
  BarcodeFormat.AZTEC: 'AZTEC',
  BarcodeFormat.CODABAR: 'CODABAR',
  BarcodeFormat.CODE_39: 'CODE_39',
  BarcodeFormat.CODE_93: 'CODE_93',
  BarcodeFormat.CODE_128: 'CODE_128',
  BarcodeFormat.DATA_MATRIX: 'DATA_MATRIX',
  BarcodeFormat.EAN_8: 'EAN_8',
  BarcodeFormat.EAN_13: 'EAN_13',
  BarcodeFormat.ITF: 'ITF',
  BarcodeFormat.PDF_417: 'PDF_417',
  BarcodeFormat.QR_CODE: 'QR_CODE',
  BarcodeFormat.RSS_14: 'RSS_14',
  BarcodeFormat.RSS_EXPANDED: 'RSS_EXPANDED',
  BarcodeFormat.UPC_A: 'UPC_A',
  BarcodeFormat.UPC_E: 'UPC_E',
  BarcodeFormat.UNKNOWN: 'UNKNOWN',
};

String imageFilterJsonValue(ImageFilterType filter) {
  return filter.toString().replaceFirst("ImageFilterType.", "");
}

const imageFormatMap = {ImageFormat.JPG: "JPG", ImageFormat.PNG: "PNG"};
const contourDetectorTypeMap = {DocumentDetectorMode.ML_BASED: "ML_BASED", DocumentDetectorMode.EDGE_BASED: "EDGE_BASED"};
